! main.m
!
! Program to compute the Stochastic Neoclassical Growth Model with a Value Function Iteration
!
! Jesus Fernandez-Villaverde
! Philadelphia, February 13th, 2003

program myopt

	use mod_types
	use mod_common_var
	use dfport
	
	implicit none

	integer, allocatable, dimension (:) :: length_grid_k, max_iterations, print_mod
	
	real(dp), allocatable, dimension(:,:)  :: transition, value, value_old
	real(dp), allocatable, dimension(:)  :: y, grid_k, grid_k_old

	integer :: multigrid, index_k, index_z

	real(dp) :: phi, omega, big_phi, k_ss, l_ss, c_ss, r_ss, y_ss
	real(dp) :: cover, cover_tauchen, interp_value
	
	real (4) :: elapt, ta(2)

	print *, ' '
	print *, 'Beginning of the Program Value Function Iteration'
	print *, ' '
	
	!----------------------------------------------------------------
	! 1. Computation of the Steady State
	!----------------------------------------------------------------

	phi     = (1/alpha*(1/beta-1+delta))**(1/(1-alpha))
	omega   = (phi**(1-alpha)-delta)
	big_phi = theta/(1-theta)*(1-alpha)*phi**(-alpha) 
  
	k_ss = big_phi/(omega+phi*big_phi)
	l_ss = phi*k_ss
	c_ss = omega*k_ss
	r_ss = alpha*(k_ss**(alpha-1))*(l_ss**(1-alpha))
	y_ss = k_ss**alpha*(l_ss**(1-alpha))

	print *, 'Steady State values'
	print *, 'Output: ', y_ss, 'Capital: ', k_ss, 'Labor: ', l_ss

	allocate(transition(n_tauchen,n_tauchen))
	allocate(y(n_tauchen))

	!----------------------------------------------------------------
	! 2. Tauchen Points
	!----------------------------------------------------------------

	cover_tauchen = 3.d0  
	call sub_tauchen(y,transition,rho,sigma,n_tauchen,cover_tauchen)

	print *, ' '
	print *, 'For the stochastic shock, Tauchen`s approximation method with', n_tauchen , 'points will be used.'
	print *, ' '

	!----------------------------------------------------------------
	! 3. Main Iteration
	!----------------------------------------------------------------
	
	print *, 'Beginning the iteration of the value function with an initial linear guess '
	print *, 'and the sup norm with a tolerance of ', toler
	print *, ' '

	! Form the first grid for k

	cover = 0.3d0         ! Coverage of the grid (+- of steady state k) 0.9 for sigma = 0.035

	allocate(length_grid_k(multigrid_steps))
	allocate(max_iterations(multigrid_steps))
	allocate(print_mod(multigrid_steps))
	
	length_grid_k(1) = 200
	length_grid_k(2) = 1000
	length_grid_k(3) = 2500
	length_grid_k(4) = 5000
	length_grid_k(5) = 25000

	max_iterations(1) = 1000
	max_iterations(2) = 1000
	max_iterations(3) = 1000
	max_iterations(4) = 1000
	max_iterations(5) = 1000

	print_mod(1) = 100
	print_mod(2) = 1
	print_mod(3) = 1
	print_mod(4) = 1
	print_mod(5) = 1

	do multigrid = 1, multigrid_steps

		allocate(grid_k(length_grid_k(multigrid)))
		allocate(value(length_grid_k(multigrid),n_tauchen))
		allocate(g_l(length_grid_k(multigrid),n_tauchen))
		allocate(g_k(length_grid_k(multigrid),n_tauchen))

		call sub_grid_generation(cover,length_grid_k(multigrid),k_ss,grid_k)

		if (multigrid==1) then

			value = (((c_ss**theta)*((1-l_ss)**theta))**(1-tau))/(1-tau)
		
		else

			do index_k = 1, length_grid_k(multigrid) 			
				do index_z = 1, n_tauchen
					call sub_myinterp(grid_k_old,value_old(:,index_z),grid_k(index_k),size(grid_k_old),interp_value)
					value(index_k, index_z) = interp_value
				enddo
			enddo
			
			print *, 'Interpolation finished'

		endif
	
		call sub_value(value,grid_k,length_grid_k(multigrid),y,transition,max_iterations(multigrid),print_mod(multigrid))

		if (multigrid<multigrid_steps) then

			if (multigrid>1) then
				deallocate(grid_k_old)
				deallocate(value_old)
			endif
			
			allocate(grid_k_old(length_grid_k(multigrid)))
			allocate(value_old(length_grid_k(multigrid),n_tauchen))
		
			grid_k_old = grid_k
			value_old = value

			deallocate(grid_k)
			deallocate(value)
			deallocate(g_l)
			deallocate(g_k)
		
		endif

	enddo

	!----------------------------------------------------------------
	! 4. Output
	!----------------------------------------------------------------
	
	open(unit=11,	file='value.txt', status = 'replace')
	write (11, '(40f20.10)') (value(i1, :), i1 = 1,length_grid_k(multigrid_steps))
	close(11)

	open(unit=12,	file='g_l.txt', status = 'replace')
	write (12, '(40f14.7)') (g_l(i1, :), i1 = 1,length_grid_k(multigrid_steps))
	close(12)
	
	open(unit=13,	file='g_k.txt', status = 'replace')
	write (13, '(40f14.7)') (g_k(i1, :), i1 = 1,length_grid_k(multigrid_steps))
	close(13)
	
	elapt = etime(ta)
	write (*,*), ' '
	write (*,'(a17, x, f8.2, x, a20)'), ' Program has used', elapt, 'seconds of CPU time,'
	write (*,'(a5, x, f8.2, x, a24, x, f5.2, x, a23)'), ' with', ta(1), 'seconds of user time and', ta(2), 'seconds of system time.'
	
	print *, ' '
	print *, 'End of the program'
	print *, ' '
	
end program myopt